package com.njyunzhi.pet_identity.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaQrcodeService;
import cn.binarywang.wx.miniapp.bean.WxMaCodeLineColor;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.njyunzhi.pet_identity.common.OSSUtils;
import com.njyunzhi.pet_identity.common.StringUtils;
import com.njyunzhi.pet_identity.common.WxUtils;
import com.njyunzhi.pet_identity.entity.TaMiniappQrcode;
import com.njyunzhi.pet_identity.mapper.TaMiniappQrcodeMapper;
import com.njyunzhi.pet_identity.service.ITaMiniappQrcodeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileNotFoundException;
import java.time.LocalDateTime;

/**
 * <p>
 * 小程序码 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-05-28
 */
@Service
public class TaMiniappQrcodeServiceImpl extends ServiceImpl<TaMiniappQrcodeMapper, TaMiniappQrcode> implements ITaMiniappQrcodeService {

    @Autowired
    WxUtils wxUtils;
    
    @Autowired
    OSSUtils ossUtils;

    @Value("${weixin.miniapp.appid}")
    String miniAppid;

    @Override
    public TaMiniappQrcode createQRCode(String page, String scene) throws Exception {
        TaMiniappQrcode taMiniappQrcode = getByPageAndScene(page, scene);
        if (null != taMiniappQrcode) return taMiniappQrcode;

        // 调用SDK
        String qrImage = buildMAQrCode(page, scene);
        taMiniappQrcode = new TaMiniappQrcode();
        taMiniappQrcode.setMiniappId(miniAppid);
        taMiniappQrcode.setPage(page);
        taMiniappQrcode.setScene(scene);
        taMiniappQrcode.setQrImage(qrImage);

        if (!save(taMiniappQrcode)) {
            throw new Exception("生成小程序码失败, 请重试");
        }

        return taMiniappQrcode;
    }

    public TaMiniappQrcode getByPageAndScene(String page, String scene) {
        LambdaQueryWrapper<TaMiniappQrcode> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaMiniappQrcode::getMiniappId, miniAppid)
                .eq(TaMiniappQrcode::getPage, page)
                .eq(!StringUtils.isEmpty(scene), TaMiniappQrcode::getScene, scene)
                .orderByDesc(TaMiniappQrcode::getCreateDate)
                .last("limit 1");

        return getOne(queryWrapper);
    }

    /**
     * 生成小程序码, 并上传阿里云
     * @param page
     * @param scene
     * @return
     * @throws WxErrorException
     */
    private String buildMAQrCode(String page, String scene) throws WxErrorException, FileNotFoundException {
        WxMaQrcodeService qrcodeService = wxUtils.getMaService().getQrcodeService();
        File qrFile = qrcodeService.createWxaCodeUnlimit(scene, page);

        return ossUtils.putObject(qrFile);
    }
}
